@@ -3,6 +3,8 @@ import json
 from qds_ops.utils.ssh_tunnel import SSHTunnel
 from subprocess import PIPE, check_call
 import os
+import random
+import socket
 
 class RDS():
   def __init__(self, subparser):
@@ -22,6 +24,8 @@ class RDS():
         action = "store_true", help="enable tab-completion of table and column names (with slower startup)")
     cn_parser.add_argument("-r", "--replica", dest="replica",
         action = "store_true", help="Login into read replica")
+    cn_parser.add_argument("-t", "--tunnel", dest="tunnel", 
+        action = "store_true", help="Use a tunnel through a web node")
     cn_parser.set_defaults(func=self.connect)
 
     #tunnel
@@ -42,6 +46,13 @@ class RDS():
 
     print json.dumps(output, indent=4, sort_keys=True)
 
+  def checkport(self, port):
+    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    result = s.connect_ex(('127.0.0.1', 3306))
+    s.close()
+
+    return result != 0
+
   def connect(self, args):
     knife = Knife()
     creds = knife.get_credentials(args.environment)
@@ -50,32 +61,33 @@ class RDS():
       hostname = creds["rr_host"]
     else:
       hostname = creds["host"]
+    port = 3306
+    pemfile = None
 
-    if args.autocomplete:
-        mysql  = check_call(["mysql", "-u", creds["username"],
-        "-p%s" % creds["password"],"-h", hostname])
-    else:
-        mysql  = check_call(["mysql", "-u", creds["username"],
-        "-p%s" % creds["password"],"-h", hostname, "-A"])
+    if args.tunnel:
+      pemfile = knife.get_pemfile(args.environment)
+      list = knife.search(args.environment, "web")
+      port = random.randrange(1,1000) + 33000
+      counter = 1
+      while not self.checkport(port) and counter < 5:
+        port = random.randrange(1,1000) + 33000
+        counter = counter + 1
 
+      if counter > 5:
+        raise RuntimeError("Could not find a free port")
+      tunnel = SSHTunnel("ec2-user", list[0], hostname, pemfile, 22, 3306, port)
+      hostname = "127.0.0.1"
+    
+    mysql_cmd = ["mysql", "-u", creds["username"],
+      "-p%s" % creds["password"],"-h", hostname, "-P", str(port), "--prompt", "%s>\_" % args.environment]
+    if not args.autocomplete:
+      mysql_cmd.append("-A")
+      
+    mysql  = check_call(mysql_cmd)
+
+    if pemfile is not None:
+      os.unlink(pemfile)
+  
   def tunnel(self, args):
-    knife = Knife()
-    temp = knife.get_pemfile(args.environment)
-    list = knife.search(args.environment, "web")
-    creds = knife.get_credentials(args.environment)
-    hostname = ""
-    if args.replica:
-      hostname = creds["rr_host"]
-    else:
-      hostname = creds["host"]
-
-    tunnel = SSHTunnel("ec2-user", list[0], hostname, temp, 22, 3306, 33306)
-
-    if args.autocomplete:
-        mysql  = check_call(["mysql", "-u", creds["username"],
-        "-p%s" % creds["password"],"-h", "127.0.0.1", "-P", "33306"])
-    else:
-        mysql  = check_call(["mysql", "-u", creds["username"],
-        "-p%s" % creds["password"],"-h", "127.0.0.1", "-P", "33306", "-A"])
-
-    os.unlink(temp)
+    args.tunnel = True
+    self.connect(args)
